#ifndef _MYMATRIXH_
#define _MYMATRIXH_
#include <iosfwd>

template <typename T>
class matrix {
private:
  int rows, cols;
  T* elements;
public:
  matrix();
  matrix(int r, int c);
  matrix(matrix<T>& m);
  matrix& operator=(matrix<T>& m);
  ~matrix();
  int get_rows();
  int get_cols();
  void fill(T value);
  T& operator() (int i, int j);
};

template <typename T>
matrix<T>::matrix() {
  rows = 0;
  cols = 0;
  elements = new T[0];
}

template <typename T>
matrix<T>::matrix(int r, int c) {
  rows = r;
  cols = c;
  elements = new T[rows*cols];
  if (elements==0)
    std::cerr << "Error: could not allocate memory\n";
}

template <typename T>
matrix<T>::matrix(matrix<T>& m) {
  rows = m.rows;
  cols = m.cols;
  elements = new T[rows*cols];
  for (int i=0; i < rows*cols; i++)
    elements[i] = m.elements[i];
}

template <typename T>
matrix<T>& matrix<T>::operator=(matrix<T>& m) {
  if (&m != this) {
    delete[] elements;
    rows = m.rows;
    cols = m.cols;
    elements = new T[rows*cols];
    for (int i=0; i<rows*cols; i++)
      elements[i] = m.elements[i];
  }
  return *this;
}

template <typename T>
matrix<T>::~matrix<T>() {
  rows = 0;
  cols = 0;
  delete[] elements;
}

template <typename T>
int matrix<T>::get_rows() {
  return rows;
}

template <typename T>
int matrix<T>::get_cols() {
  return cols;
}

template <typename T>
T& matrix<T>::operator()(int i, int j) {
  return elements[i*cols + j];
}

template <typename T>
void matrix<T>::fill(T value) {
  for (int i=0; i < rows; i++) 
    for (int j=0; j < cols; j++) 
      (*this)(i,j)=value;
}

template <typename T>
std::ostream& operator<<(std::ostream& o, matrix<T>& m) {
  for (int i=0; i<m.get_rows(); i++) {
    for (int j=0; j<m.get_cols(); j++) 
      o << m(i,j) << " ";
    o << "\n";
  }
}


#endif
